/*

    Zelda Time to Triumph

    Copyright (C) 2007-2009  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include <SDL2/SDL.h>

#include <iostream>
#include <cstdlib>
#include <ctime>

#include "Keyboard.h"
#include "Generique.h"


SDL_Window* initWindow() {            // initialise SDL
    if(SDL_Init(SDL_INIT_VIDEO) == -1) {
        printf("Could not load SDL : %s\n", SDL_GetError());
        exit(-1);
    }

    SDL_Window* window = SDL_CreateWindow("Time to Triumph",
                      SDL_WINDOWPOS_CENTERED,
                      SDL_WINDOWPOS_CENTERED,
                      640,
                      480,
                      SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE | SDL_WINDOW_FULLSCREEN_DESKTOP);



    SDL_Surface* icon = SDL_LoadBMP("data/images/logos/ocarina.ico");
    SDL_SetColorKey(icon, SDL_TRUE , SDL_MapRGB(icon->format,0,0,0));
    SDL_SetWindowIcon(window, icon);
    SDL_FreeSurface(icon);

    SDL_ShowCursor(SDL_DISABLE);

    return window;
}

SDL_Renderer* initRenderer(SDL_Window* window) {
    return SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);
}

SDL_Texture* init(SDL_Renderer* renderer) {
    SDL_Texture* screen = SDL_CreateTexture(renderer, SDL_PIXELFORMAT_RGBA8888, SDL_TEXTUREACCESS_TARGET, 320, 240);
    SDL_SetRenderTarget(renderer, screen);
    SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
    SDL_RenderClear(renderer);
    return screen;
}

int main(int argc, char** argv) {
    if (argc && argv); //pour viter un warning.....

    std::srand(std::time(NULL));

    SDL_Window* window = initWindow();
    SDL_Renderer* renderer = initRenderer(window);
    SDL_Texture* gpScreen = NULL;

    int mode = 2; //mode=1;



    gpScreen = init(renderer);
    SDL_Surface* gpScreen2 = SDL_CreateRGBSurface(0, 320, 240, 32, 0, 0, 0, 0);

    Audio* gpAudio = new Audio();
    Jeu* gpJeu = new Jeu(gpAudio);
    Carte* gpCarte = new Carte(gpJeu);
    Encyclopedie* gpEncyclopedie = new Encyclopedie(gpJeu);
    Poissons* gpPoissons = new Poissons(gpJeu);
    Keyboard* gpKeyboard = new Keyboard(gpJeu, gpCarte, gpEncyclopedie, gpPoissons, window, mode);
    gpJeu->setKeyboard(gpKeyboard);
    Generique* gpGenerique = new Generique(gpJeu);
    gpJeu->setGenerique(gpGenerique);
    gpGenerique->initLogo();

    //gpJeu->init(0); // virer

    bool gLoop = true;

    Uint32 lastAnimTime = SDL_GetTicks();

    while (gLoop) {

        if (gpKeyboard->gererClavier() == -1) {gLoop = false;}

        switch (gpKeyboard->getMode()) {
            case 0 : //jeu normal
                gpJeu->draw(gpScreen2); break;
            case 1 : //disclamer
            case 2 : //logo
            case 3 : //titre
            case 14 : //gnrique score
            case 17 : //menu d'aide 1
            case 18 : //menu d'aide 2
            case 24 : //menu d'aide 3
            case 25 : //items de troc
            case 28 : //image de fin
                gpGenerique->draw(gpScreen2); break;
            case 4 : //selection
                gpGenerique->drawSelection(gpScreen2, gpKeyboard->getLigne(),
                    gpKeyboard->getColonne()); break;
            case 6 : //options
                gpGenerique->drawOption(gpScreen2, gpKeyboard->getLigneOption(),
                gpKeyboard->getVolume()/8, gpKeyboard->getVolson()/8); break;
            case 7 : //charger partie
                gpGenerique->drawCharger(gpScreen2, gpKeyboard->getLigne(),
                    gpKeyboard->getLigneVal()); break;
            case 8 : //gnrique intro
                gpGenerique->drawIntro(gpScreen2, gpKeyboard->getIntro()); break;
            case 9 : //effacer partie
                gpGenerique->drawEffacerSave(gpScreen2, gpKeyboard->getLigne(),
                    gpKeyboard->getLigneVal()); break;
            case 10 : //gnrique dbut chez link
                gpGenerique->drawDebut(gpScreen2); break;
            case 11 : //gnrique fin
                gpGenerique->drawFin(gpScreen2); break;
            case 12 : //carte
            case 22 : //carte tlportation
                gpCarte->draw(gpScreen2); break;
            case 13 : //encyclopdie des monstres
                gpEncyclopedie->draw(gpScreen2); break;
            case 15 : //records
            case 19 : //rang 100%
            case 20 : //rang ultime
            case 21 : //rang de rapidit
                gpGenerique->drawRecord(gpScreen2, gpKeyboard->getLigneRecord(),
                    gpKeyboard->getColonneRecord()); break;
            case 16 : //effacer record
                gpGenerique->drawEffacer(gpScreen2, gpKeyboard->getLigneVal()); break;
            case 23 : //encyclopdie des poissons
                gpPoissons->draw(gpScreen2); break;
            case 26 : //gnrique vers prsent
            case 27 : //gnrique vers pass
                gpGenerique->drawToPresent(gpScreen2); break;
            default : break;
        }



        SDL_Texture* texture = SDL_CreateTextureFromSurface(renderer, gpScreen2);
        SDL_RenderCopy(renderer, texture, NULL, NULL);
        SDL_DestroyTexture(texture);

        SDL_SetRenderTarget(renderer, NULL);
        SDL_RenderCopy(renderer, gpScreen, NULL, NULL);

        int rw, rh;
        SDL_GetWindowSize(window, &rw, &rh);
        SDL_RenderSetScale( renderer, ((double)rw) / ((double)320), ((double)rh) / ((double)240));

        SDL_RenderPresent(renderer);
        SDL_RenderClear(renderer);
        if (SDL_GetTicks() < lastAnimTime + 20) SDL_Delay(lastAnimTime + 20 - SDL_GetTicks());
        lastAnimTime = SDL_GetTicks();

        SDL_SetRenderTarget(renderer, gpScreen);

    }

    SDL_FreeSurface(gpScreen2);

    delete gpPoissons;
    delete gpEncyclopedie;
    delete gpGenerique;
    delete gpKeyboard;
    delete gpCarte;
    delete gpJeu;
    delete gpAudio;

    SDL_SetRenderTarget(renderer, NULL);
    SDL_DestroyTexture(gpScreen);
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_ShowCursor(SDL_ENABLE);
    SDL_Quit();
    exit(0);

    return 0;
}
